import React from "react";

const localStorageKey = "usr_jwt";
interface AppProviderProps {
  type: "regular" | "firebase";
  token: string | null;
  setToken: (newToken: string) => void
}

const AppContext = React.createContext<AppProviderProps>({type: 'regular', token: null, setToken: () => {}});

export function AppProvider(props: AppProviderProps) {
  const [type] = React.useState(props.type);
  const [token, setToken] = React.useState<string|null>(props.token);

  React.useEffect(() => {
    const jwt = localStorage.getItem(localStorageKey);
    setToken(jwt);
  }, []);

  React.useEffect(() => {
    if (token && token !== "") {
      localStorage.setItem(localStorageKey, token);
    } else if (localStorage.getItem(localStorageKey)) {
      localStorage.removeItem(localStorageKey);
    }
  }, [token]);

  const memData = React.useMemo(() => {
    return { token, type, setToken };
  }, [token, type, setToken]);

  return <AppContext.Provider value={memData} {...props} />;
}

export function useApp() {
  const context = React.useContext(AppContext);
  if (!context) {
    // eslint-disable-next-line no-throw-literal
    throw "error: app context not defined.";
  }
  return context;
}
